local _,L = ...
local rematch = Rematch
local panel = RematchTeamTabs
local dialog, iconPicker, settings, saved

local MAX_TABS = 12

-- returns a texture with Interface\Icons\ removed
local function filename(iconPath) return iconPath:gsub("[iI][nN][tT][eE][rR][fF][aA][cC][eE]\\[iI][cC][oO][nN][sS]\\","") end

rematch:InitModule(function()
	rematch.TeamTabs = panel
	settings = RematchSettings
	saved = RematchSaved
	dialog = RematchDialog
	iconPicker = dialog.TeamTabIconPicker
	panel.Tabs[1]:SetPoint("TOPLEFT",2,0)
	for i=1,MAX_TABS do
		if i>1 then
			panel.Tabs[i]:SetPoint("TOPLEFT",panel.Tabs[i-1],"TOPLEFT",0,-48)
		end
		panel.Tabs[i]:RegisterForClicks("AnyUp")
	end

	local function isFirstTab(self,index) return index==1 end

	rematch:RegisterMenu("TabMenu", { -- menu for Tab buttons
		{ title=true, text=function(self,index) return settings.TeamGroups[index][1] end, },
		{ text=L["Edit Tab"], func=panel.ShowTabEditDialog, tooltipBody=L["Change the name or icon of this tab."] },
		{ text=L["Custom Sort"], check=true,
				value=function(entry,index)
					return settings.TeamGroups[index] and settings.TeamGroups[index][3] and true
				end,
				func=function(entry,index)
					local tab = settings.TeamGroups[index]
					if not tab then return end
					if tab[3] then
						local dialog = rematch:ShowDialog("DeleteCustomSort",300,148,L["Remove Custom Sort"],nil,YES,function() tab[3]=nil rematch:UpdateUI() end,NO)
						dialog:ShowText(L["Are you sure you want to sort teams alphabetically in this tab?\n\nThe custom sort order will be lost and cannot be undone."],240,80,"TOP",0,-32)
					else
						tab[3] = {}
					end
					rematch:HideMenu()
					rematch:UpdateUI()
				end,
				tooltipBody=L["While checked, you can rearrange the order of teams within this tab."] },
		{ text=L["Export Tab"], func=function() Rematch.Dialog.Share:ExportTeamTab() end, tooltipBody=L["Export all teams in this tab to a string you can copy elsewhere, such as forums or emails.\n\nOther Rematch users can then paste these teams into their Rematch via Import Teams.\n\nYou can export a single team by right-clicking one and choosing its Share menu."] },
		{ text=L["Import Teams"], func=rematch.ShowImportDialog, tooltipBody=L["Import a single team or many teams that was exported from Rematch."] },
		{ text=L["Delete Tab"], hidden=isFirstTab, func=panel.ShowTabDeleteDialog, tooltipBody=L["Delete this tab and move all of its teams to the default tab."] },
		{ text=L["Move Up"], hidden=isFirstTab, icon="Interface\\Buttons\\UI-MicroStream-Yellow", iconCoords={0.075,0.925,0.925,0.075}, stay=true,
			disabled=function(entry,index) return index==2 end,
			func=function(entry,index) panel:SwapTabs(index,index-1) end },
		{ text=L["Move Down"], hidden=isFirstTab, icon="Interface\\Buttons\\UI-MicroStream-Yellow", stay=true,
			disabled=function(entry,index) return index==#settings.TeamGroups end,
			func=function(entry,index) panel:SwapTabs(index,index+1) end },
		{ text=OKAY },
	})

end)

-- returns the selected team tab (and assigns it to first tab if anything weird happened)
function panel:GetSelectedTab()
	local selected = settings.SelectedTab
	if not selected or not settings.TeamGroups[selected] then
		settings.SelectedTab = 1
		return 1
	else
		return selected
	end
end

function panel:Update()
	local groups = settings.TeamGroups

	panel:GetSelectedTab() -- changes settings.SelectedTab to 1 if selected tab doesn't eixt
	local searchText,searchTabs = rematch.TeamPanel:GetTeamSearchInfo()
	local tabs = panel.Tabs
	for i=1,MAX_TABS do
		if groups[i] then
			tabs[i].Icon:SetTexture(groups[i][2])
			tabs[i].Icon:SetDesaturated(searchText and not searchTabs[i])
			if searchText and not searchTabs[i] then
				tabs[i].Icon:SetVertexColor(0.5,0.5,0.5)
			else
				tabs[i].Icon:SetVertexColor(1,1,1)
			end
			tabs[i]:Show()
			tabs[i].Custom:SetShown(groups[i][3] and true)
		elseif i==#groups+1 then
			tabs[i].Icon:SetTexture("Interface\\GuildBankFrame\\UI-GuildBankFrame-NewTab")
			tabs[i].Custom:Hide()
			tabs[i]:Show()
		else
			tabs[i]:Hide()
		end
		tabs[i]:SetChecked(settings.SelectedTab==i and not searchText)
	end

	if #groups<10 then
		tabs[1]:SetPoint("TOPLEFT",2,0)
	elseif #groups==10 then
		tabs[1]:SetPoint("TOPLEFT",2,32)
	else
		tabs[1]:SetPoint("TOPLEFT",2,64)
	end

end

function panel:SelectTeamTab(index)
	rematch.TeamPanel:SetTeamSearch(nil)
	settings.SelectedTab = index or 1
	panel:Update()
	if not rematch.TeamPanel:IsVisible() and not rematch:IsDialogOpen("SaveAs") and not rematch:IsDialogOpen("ImportDialog") and not rematch:IsDialogOpen("ReceiveDialog") then
		rematch:ShowTeam() -- if teams not shown then switch to panel
	else
		rematch.TeamPanel:Update()
	end
	rematch:ListScrollToTop(rematch.TeamPanel.List.ScrollFrame)
	rematch.Dialog:UpdateTabPicker()
end

function panel:TabOnClick(button)
	rematch:HideMenu()
	local index = self:GetID()
	local groups = settings.TeamGroups
	if button=="RightButton" then
		if index<=#groups then
			rematch:SetMenuSubject(index)
			panel:SelectTeamTab(index)
			rematch:ShowMenu("TabMenu","cursor")
		else
			self:SetChecked(false)
		end
		return
	end
	if index>0 and index<=#groups then
		panel:SelectTeamTab(index)
	else
		self:SetChecked(false)
		if index>#groups then
			panel:ShowTabEditDialog(index)
		end
	end
end

function panel:TabOnEnter()
	local index = self:GetID()
	if index>#settings.TeamGroups then
		rematch.ShowTooltip(self,L["Create New Tab"])
	else
		local numTeams = 0
		for k,v in pairs(saved) do
			if (v.tab or 1)==index then
				numTeams = numTeams + 1
			end
		end
		rematch.ShowTooltip(self,settings.TeamGroups[index][1],format(L["Teams: %d"],numTeams))
	end
end


function panel:ShowTabEditDialog(index)
	local groups = settings.TeamGroups
	local name, icon
	if index>#groups then
		name = L["New Tab"]
		icon = "Interface\\Icons\\PetJournalPortrait"
	else
		name = groups[index][1]
		icon = groups[index][2]
	end
	rematch:ShowDialog("TabEdit",300,346,name or L["New Tab"],L["Choose a name and icon"],SAVE,panel.IconPickerSave,CANCEL)
	dialog:SetContext("tab",index)
	dialog:SetContext("name",name)
	dialog:SetContext("icon",icon)

	dialog.EditBox:SetPoint("TOP",22,-40)
	dialog.EditBox:SetWidth(180)
	dialog.EditBox:Show()
	dialog.EditBox:SetText(name)

	dialog.Slot:SetPoint("RIGHT",dialog.EditBox,"LEFT",-8,0)
	dialog.Slot:Show()
	dialog.Slot.Icon:SetTexture(icon)

	iconPicker:SetPoint("TOP",0,-80)
	if not iconPicker.ScrollFrame.buttons then
		-- scrollframe hasn't been set up yet
		local scrollFrame = iconPicker.ScrollFrame
		scrollFrame.update = panel.UpdateTabIconPickerList
		scrollFrame.ScrollBar.doNotHide = true
		scrollFrame.stepSize = 120
		HybridScrollFrame_CreateButtons(scrollFrame,"RematchTeamTabPickerListTemplate")
	end
	-- set up icon choices (recreating table each time on purpose; don't want to keep these huge tables)
	local iconChoices = dialog:SetContext("iconChoices",{})
	tinsert(iconChoices,"PetJournalPortrait")
	GetLooseMacroIcons(iconChoices)
	GetMacroIcons(iconChoices)
	GetMacroItemIcons(iconChoices)
	iconPicker.CurrentIcon:SetText(icon:gsub("Interface\\Icons\\",""))
	panel:UpdateTabIconPickerList()
	iconPicker:Show()
end

-- when save clicked on IconPicker
function panel:IconPickerSave()
	local groups = settings.TeamGroups
	local index = dialog:GetContext("tab")
	groups[index] = groups[index] or {}
	groups[index][1] = dialog.EditBox:GetText() or ""
	groups[index][2] = dialog:GetContext("icon")
	panel:SelectTeamTab(index)
end

-- update of icon list in iconpicker
function panel:UpdateTabIconPickerList()
	-- data points to searchChoices if a search is used, or iconChoices (all icons) otherwise
	local data = dialog:GetContext("search") and dialog:GetContext("searchChoices") or dialog:GetContext("iconChoices")
	if not data then return end
	local numData = ceil(#data/7)
	local scrollFrame = iconPicker.ScrollFrame
	local offset = HybridScrollFrame_GetOffset(scrollFrame)
	local buttons = scrollFrame.buttons
	local selectedIcon = dialog:GetContext("icon")
	for i=1,#buttons do
		local index = i + offset
		local button = buttons[i]
		if ( index <= numData) then
			for j=1,7 do
				local subbutton = button.Icons[j]
				local iconFilename = data[(index-1)*7+j]
				if iconFilename then
					if type(iconFilename)=="number" then
						subbutton.Texture:SetToFileData(iconFilename)
					else
						subbutton.Texture:SetTexture("Interface\\Icons\\"..iconFilename)
					end
					subbutton.Selected:SetShown(subbutton.Texture:GetTexture()==selectedIcon)
					subbutton:Show()
					if GetMouseFocus()==subbutton then -- update CurrentIcon label if mouse is over a button being scrolled
						subbutton:GetScript("OnEnter")(subbutton)
					end
				else
					subbutton:Hide()
				end
			end
			button:Show()
		else
			button:Hide()
		end
	end
	HybridScrollFrame_Update(scrollFrame,30*numData,30)
end

function panel:IconOnEnter()
	iconPicker.CurrentIcon:SetText(filename(self.Texture:GetTexture()))
end

function panel:IconOnLeave()
	iconPicker.CurrentIcon:SetText(filename(dialog:GetContext("icon")))
end

function panel:IconOnClick()
	local texture = self.Texture:GetTexture()
	if texture then
		dialog:SetContext("icon",texture)
		dialog.Slot.Icon:SetTexture(texture)
		panel:UpdateTabIconPickerList()
	end
end

function panel:SearchOnTextChanged()
	local text = self:GetText()
	local wait = 0
	if text and text:trim():len()>0 then
		dialog:SetContext("search",text)
		wait = 0.5
	else
		dialog:SetContext("search",nil)
	end
	-- unless search emptied do a delayed update on list (so let user type a bit before searching all billion icons)
	rematch:StartTimer("TabIconPicker",wait,panel.DelayedSearchOnTextChanged)
end

function panel:DelayedSearchOnTextChanged()
	local search = dialog:GetContext("search")
	if search then
		local testIcon = iconPicker.Search.TestIcon
		local searchChoices = dialog:GetContext("searchChoices") or dialog:SetContext("searchChoices",{})
		wipe(searchChoices)
		search = rematch:DesensitizeText(search)
		for _,candidate in ipairs(dialog:GetContext("iconChoices")) do
			if type(candidate)=="number" then -- this is a fileid
				testIcon:SetToFileData(candidate) -- set it to the test texture
				candidate = filename(testIcon:GetTexture() or "") -- and GetTexture to find out what it is
			else
				candidate = filename(candidate)
			end
			if candidate:match(search) then
				tinsert(searchChoices,candidate)
			end
		end
	end
	panel:UpdateTabIconPickerList()
end

--[[ Delete ]]

function panel:ShowTabDeleteDialog(index)
	local groups = settings.TeamGroups
	rematch:ShowDialog("TabDelete",300,150,groups[index][1],L["Delete this tab?"],YES,panel.ConfirmDeleteTab,NO)
	dialog:SetContext("tab",index)
	dialog:ShowText(format(L["Deleting this tab will move its teams to the %s tab."],groups[1][1]),180,40,"TOP",22,-40)
	dialog.Slot:SetPoint("RIGHT",dialog.Text,"LEFT",-8,0)
	dialog.Slot:Show()
	dialog.Slot.Icon:SetTexture(groups[index][2])
end

function panel:ConfirmDeleteTab()
	local index = dialog:GetContext("tab")
	if index then
		tremove(settings.TeamGroups,index)
		panel:SelectTeamTab(1) -- go to first tab (this will update tabs and team panel)
	end
end

-- swaps two tabs (index 2 to #groups) for moving tabs up/down
function panel:SwapTabs(index1,index2)
	local groups = settings.TeamGroups
	if index1==1 or index2==1 or index1>#groups or index2>#groups then
		return -- don't swap first tab!
	end
	for teamName,teamData in pairs(saved) do
		local teamTab = teamData.tab
		if teamTab==index1 then
			teamData.tab = index2
		elseif teamTab==index2 then
			teamData.tab = index1
		end
	end
	local name,icon,custom = groups[index1][1], groups[index1][2], groups[index1][3]
	groups[index1][1] = groups[index2][1]
	groups[index1][2] = groups[index2][2]
	groups[index1][3] = groups[index2][3]
	groups[index2][1] = name
	groups[index2][2] = icon
	groups[index2][3] = custom
	rematch:SetMenuSubject(index2) -- change menu subject to destination tab
	panel:SelectTeamTab(index2)
end

function panel:Reparent(parent)
	if parent~=panel:GetParent() then
		panel:SetParent(parent)
		panel:ClearAllPoints()
		if settings.TeamTabsToLeft and settings.AlwaysTeamTabs and parent==rematch.Frame then
			panel:SetPoint("TOPRIGHT",parent,"TOPLEFT",4,-100)
			for i=1,12 do
				panel.Tabs[i].Back:SetPoint("TOPLEFT",-30,12)
				panel.Tabs[i].Back:SetTexCoord(1,0,0,1)
			end
		else
			panel:SetPoint("TOPLEFT",parent,"TOPRIGHT",0,-100)
			for i=1,12 do
				panel.Tabs[i].Back:SetPoint("TOPLEFT",-4,12)
				panel.Tabs[i].Back:SetTexCoord(0,1,0,1)
			end
		end
	end
	panel:Show()
end